import sys
import json
import inspect
from enum import Enum, IntEnum
from typing import Any, List, Union, Literal, Optional

from pydantic import Field, BaseModel, validator, root_validator


class ApiResponse(BaseModel):
    retcode: int
    message: str
    data: Any


class BotAuth(BaseModel):
    bot_id: str
    bot_secret: str


# http事件回调部分
# see https://webstatic.mihoyo.com/vila/bot/doc/callback.html
class RobotCommand(BaseModel):
    name: str
    desc: str


class RobotTemplate(BaseModel):
    id: str
    name: str
    desc: str
    icon: str
    commands: Optional[List[RobotCommand]] = None


class Robot(BaseModel):
    villa_id: int
    template: RobotTemplate


class Payload(BaseModel):
    robot: Robot
    type: int
    id: str
    created_at: int
    send_at: int
    extend_data: dict

    @root_validator(pre=True)
    def _add_villa_id_to_extend_data(cls, values: dict):
        """把villa_id和bot_id添加到extend_data中，方便使用"""
        if values.get("type") == 2 and "SendMessage" in values.get(
            "extend_data", {}
        ).get("EventData", {}):
            if isinstance(
                values["extend_data"]["EventData"]["SendMessage"]["content"], str
            ):
                values["extend_data"]["EventData"]["SendMessage"][
                    "content"
                ] = json.loads(
                    values["extend_data"]["EventData"]["SendMessage"]["content"]
                )
            if (
                "villa_id" in values.get("robot", {})
                and "villa_id" not in values["extend_data"]["EventData"]["SendMessage"]
            ):
                values["extend_data"]["EventData"]["SendMessage"]["villa_id"] = values[
                    "robot"
                ]["villa_id"]
            if (
                "id" in values.get("robot", {}).get("template", {})
                and "bot_id" not in values["extend_data"]["EventData"]["SendMessage"]
            ):
                values["extend_data"]["EventData"]["SendMessage"]["bot_id"] = values[
                    "robot"
                ]["template"]["id"]
        return values


## 鉴权部分
## see https://webstatic.mihoyo.com/vila/bot/doc/auth_api/
class BotMemberAccessInfo(BaseModel):
    uid: int
    villa_id: int
    member_access_token: str
    bot_tpl_id: str


class CheckMemberBotAccessTokenReturn(BaseModel):
    access_info: BotMemberAccessInfo
    member: "Member"


# 大别野部分
# see https://webstatic.mihoyo.com/vila/bot/doc/villa_api/
class Villa(BaseModel):
    villa_id: int
    name: str
    villa_avatar_url: str
    onwer_uid: int
    is_official: bool
    introduce: str
    category_id: int
    tags: List[str]


# 用户部分
# see https://webstatic.mihoyo.com/vila/bot/doc/member_api/
class MemberBasic(BaseModel):
    uid: int
    nickname: str
    introduce: str
    avatar: int
    avatar_url: str


class Member(BaseModel):
    basic: MemberBasic
    role_id_list: List[int]
    joined_at: int
    role_list: List["MemberRole"]


class MemberListReturn(BaseModel):
    list: List[Member]
    next_offset: int


# 消息部分
# see https://webstatic.mihoyo.com/vila/bot/doc/message_api/
class MentionType(IntEnum):
    ALL = 1
    PART = 2

    def __repr__(self) -> str:
        return self.name


class MentionedRobot(BaseModel):
    type: Literal["mentioned_robot"] = "mentioned_robot"
    bot_id: str

    bot_name: str = Field(exclude=True)


class MentionedUser(BaseModel):
    type: Literal["mentioned_user"] = "mentioned_user"
    user_id: str

    user_name: str = Field(exclude=True)


class MentionedAll(BaseModel):
    type: Literal["mention_all"] = "mention_all"

    show_text: str = Field(exclude=True)


class VillaRoomLink(BaseModel):
    type: Literal["villa_room_link"] = "villa_room_link"
    villa_id: str
    room_id: str

    room_name: str = Field(exclude=True)


class Link(BaseModel):
    type: Literal["link"] = "link"
    url: str

    show_text: str = Field(exclude=True)


class TextEntity(BaseModel):
    offset: int
    length: int
    entity: Union[MentionedRobot, MentionedUser, MentionedAll, VillaRoomLink, Link]


class ImageSize(BaseModel):
    width: int
    height: int


class Image(BaseModel):
    url: str
    size: Optional[ImageSize] = None
    file_size: Optional[int] = None


class MessageContent(BaseModel):
    text: str
    entities: List[TextEntity]
    images: Optional[List[Image]] = None


class MentionedInfo(BaseModel):
    type: MentionType
    user_id_list: List[str] = Field(default_factory=list, alias="userIdList")


class QuoteInfo(BaseModel):
    quoted_message_id: str
    quoted_message_send_time: int
    original_message_id: str
    original_message_send_time: int


class User(BaseModel):
    portrait_uri: str = Field(alias="portraitUri")
    extra: dict
    name: str
    alias: str
    id: str
    portrait: str

    @validator("extra", pre=True)
    def extra_str_to_dict(cls, v: Any):
        if isinstance(v, str):
            return json.loads(v)
        return v


class Trace(BaseModel):
    visual_room_version: str
    app_version: str
    action_type: int
    bot_msg_id: str
    client: str
    rong_sdk_version: str


class MessageContentInfo(BaseModel):
    content: MessageContent
    mentioned_info: Optional[MentionedInfo] = Field(None, alias="mentionedInfo")
    quote: Optional[QuoteInfo] = None
    user: Optional[User] = None
    trace: Optional[Trace] = None


# 房间部分
# see https://webstatic.mihoyo.com/vila/bot/doc/room_api/
class Room(BaseModel):
    room_id: int
    room_name: str
    room_type: "RoomType"
    group_id: int
    room_default_notify_type: "RoomDefaultNotifyType"
    send_msg_auth_range: "SendMsgAuthRange"


class RoomType(str, Enum):
    CHAT = "BOT_PLATFORM_ROOM_TYPE_CHAT_ROOM"
    POST = "BOT_PLATFORM_ROOM_TYPE_POST_ROOM"
    SCENE = "BOT_PLATFORM_ROOM_TYPE_SCENE_ROOM"
    INVALID = "BOT_PLATFORM_ROOM_TYPE_INVALID"

    def __repr__(self) -> str:
        return self.name


class RoomDefaultNotifyType(str, Enum):
    NOTIFY = "BOT_PLATFORM_DEFAULT_NOTIFY_TYPE_NOTIFY"
    IGNORE = "BOT_PLATFORM_DEFAULT_NOTIFY_TYPE_IGNORE"
    INVALID = "BOT_PLATFORM_DEFAULT_NOTIFY_TYPE_INVALID"

    def __repr__(self) -> str:
        return self.name


class SendMsgAuthRange(BaseModel):
    is_all_send_msg: bool
    roles: List[int]


class GroupRoom(BaseModel):
    group_id: int
    group_name: str
    room_list: "ListRoom"


class ListRoomType(IntEnum):
    CHAT = 1
    POST = 2
    SCENE = 3

    def __repr__(self) -> str:
        return self.name


class CreateRoomType(IntEnum):
    CHAT = 1
    POST = 2
    SCENE = 3

    def __repr__(self) -> str:
        return self.name


class CreateRoomDefaultNotifyType(IntEnum):
    NOTIFY = 1
    IGNORE = 2

    def __repr__(self) -> str:
        return self.name


class ListRoom(BaseModel):
    room_id: int
    room_name: str
    room_type: ListRoomType
    group_id: int


class Group(BaseModel):
    group_id: int
    group_name: str


class RoomSort(BaseModel):
    room_id: int
    group_id: int


# 身份组部分
# see https://webstatic.mihoyo.com/vila/bot/doc/role_api/
class MemberRole(BaseModel):
    id: int
    name: str
    villa_id: int
    color: str
    web_color: str
    permissions: Optional[List["Permission"]] = None
    role_type: "RoleType"


class PermissionDetail(BaseModel):
    key: str
    name: str
    describe: str


class MemberRoleDetail(BaseModel):
    id: int
    name: str
    color: str
    villa_id: int
    role_type: "RoleType"
    member_num: int
    permissions: Optional[List[PermissionDetail]] = None


class RoleType(str, Enum):
    ALL_MEMBER = "MEMBER_ROLE_TYPE_ALL_MEMBER"
    ADMIN = "MEMBER_ROLE_TYPE_ADMIN"
    OWNER = "MEMBER_ROLE_TYPE_OWNER"
    CUSTOM = "MEMBER_ROLE_TYPE_CUSTOM"
    UNKNOWN = "MEMBER_ROLE_TYPE_UNKNOWN"

    def __repr__(self) -> str:
        return self.name


class Permission(str, Enum):
    MENTION_ALL = "mention_all"
    RECALL_MESSAGE = "recall_message"
    PIN_MESSAGE = "pin_message"
    MANAGE_MEMBER_ROLE = "manage_member_role"
    EDIT_VILLA_INFO = "edit_villa_info"
    MANAGE_GROUP_AND_ROOM = "manage_group_and_room"
    VILLA_SILENCE = "villa_silence"
    BLACK_OUT = "black_out"
    HANDLE_APPLY = "handle_apply"
    MANAGE_CHAT_ROOM = "manage_chat_room"
    VIEW_DATA_BOARD = "view_data_board"
    MANAGE_CUSTOM_EVENT = "manage_custom_event"
    LIVE_ROOM_ORDER = "live_room_order"
    MANAGE_SPOTLIGHT_COLLECTION = "manage_spotlight_collection"

    def __repr__(self) -> str:
        return self.name


class Color(str, Enum):
    GREY = "#6173AB"
    PINK = "#F485D8"
    RED = "#F47884"
    ORANGE = "#FFA54B"
    GREEN = "#7ED321"
    BLUE = "#59A1EA"
    PURPLE = "#977EE1"


# 表态表情部分
# see https://webstatic.mihoyo.com/vila/bot/doc/emoticon_api/
class Emoticon(BaseModel):
    emoticon_id: int
    describe_text: str
    icon: str


for name, obj in inspect.getmembers(sys.modules[__name__]):
    if inspect.isclass(obj) and issubclass(obj, BaseModel):
        obj.update_forward_refs()


__all__ = [
    "ApiResponse",
    "BotAuth",
    "RobotCommand",
    "RobotTemplate",
    "Robot",
    "Payload",
    "BotMemberAccessInfo",
    "CheckMemberBotAccessTokenReturn",
    "Villa",
    "MemberBasic",
    "Member",
    "MemberListReturn",
    "MentionType",
    "MentionedRobot",
    "MentionedUser",
    "MentionedAll",
    "VillaRoomLink",
    "Link",
    "TextEntity",
    "MessageContent",
    "MentionedInfo",
    "QuoteInfo",
    "User",
    "Trace",
    "ImageSize",
    "Image",
    "MessageContentInfo",
    "Room",
    "RoomType",
    "RoomDefaultNotifyType",
    "SendMsgAuthRange",
    "GroupRoom",
    "ListRoomType",
    "CreateRoomType",
    "CreateRoomDefaultNotifyType",
    "ListRoom",
    "Group",
    "RoomSort",
    "MemberRole",
    "PermissionDetail",
    "MemberRoleDetail",
    "RoleType",
    "Permission",
    "Color",
    "Emoticon",
]



from typing import TYPE_CHECKING, Any, Dict, List, Union, Optional

from nonebot.typing import overrides
from nonebot.message import handle_event
from nonebot.internal.adapter.adapter import Adapter

from nonebot.adapters import Bot as BaseBot

from .event import Event, SendMessageEvent
from .message import Message, MessageSegment
from .api import (
    Link,
    Image,
    Robot,
    ApiClient,
    ImageSize,
    QuoteInfo,
    TextEntity,
    MentionType,
    MentionedAll,
    RobotCommand,
    MentionedInfo,
    MentionedUser,
    VillaRoomLink,
    MentionedRobot,
    MessageContent,
    MessageContentInfo,
)

if TYPE_CHECKING:
    from .adapter import Adapter


async def _check_reply(bot: "Bot", event: SendMessageEvent):
    """检查事件是否有引用消息，如果有则设置 reply 字段。

    但是目前并没有API能获取被引用的消息的内容，所以现在不做。

    参数:
        bot: Bot对象
        event: 事件
    """
    ...


def _check_at_me(bot: "Bot", event: SendMessageEvent):
    """检查事件是否和机器人有关，如果有关则设置 to_me 为 True，并删除消息中的 at 信息。

    参数:
        bot: Bot对象
        event: 事件

    """
    if (
        event.content.mentioned_info
        and bot.self_id in event.content.mentioned_info.user_id_list
    ):
        event.to_me = True

    def _is_at_me_seg(segment: MessageSegment) -> bool:
        return (
            segment.type == "mentioned_robot"
            and segment.data.get("bot_id") == bot.self_id
        )

    message = event.get_message()
    if not message:
        message.append(MessageSegment.text(""))

    deleted = False
    if _is_at_me_seg(message[0]):
        message.pop(0)
        deleted = True
        if message and message[0].type == "text":
            message[0].data["text"] = message[0].data["text"].lstrip("\xa0").lstrip()
            if not message[0].data["text"]:
                del message[0]

    if not deleted:
        # check the last segment
        i = -1
        last_msg_seg = message[i]
        if (
            last_msg_seg.type == "text"
            and not last_msg_seg.data["text"].strip()
            and len(message) >= 2
        ):
            i -= 1
            last_msg_seg = message[i]

        if _is_at_me_seg(last_msg_seg):
            deleted = True
            del message[i:]

    if not message:
        message.append(MessageSegment.text(""))


class Bot(BaseBot, ApiClient):
    """
    大别野协议 Bot 适配。
    """

    @overrides(BaseBot)
    def __init__(
        self, adapter: Adapter, self_id: str, bot_info: Robot, bot_secret: str
    ):
        super().__init__(adapter, self_id)
        self.adapter: Adapter = adapter
        self.bot_secret: str = bot_secret
        self._bot_info: Robot = bot_info

    @overrides(BaseBot)
    def __repr__(self) -> str:
        return f"Bot(type={self.type!r}, self_id={self.self_id!r})"

    @property
    def nickname(self) -> str:
        """Bot 昵称"""
        return self._bot_info.template.name

    @property
    def commands(self) -> Optional[List[RobotCommand]]:
        """Bot 命令预设命令列表"""
        return self._bot_info.template.commands

    @property
    def description(self) -> str:
        """Bot 介绍描述"""
        return self._bot_info.template.desc

    @property
    def avatar_icon(self) -> str:
        """Bot 头像图标地址"""
        return self._bot_info.template.icon

    @property
    def current_villd_id(self) -> int:
        return self._bot_info.villa_id

    async def handle_event(self, event: Event):
        """处理事件"""
        if isinstance(event, SendMessageEvent):
            _check_at_me(self, event)
            # await _check_reply(self, event)
        await handle_event(self, event)

    def get_authorization_header(
        self, villa_id: Optional[int] = None
    ) -> Dict[str, str]:
        """Bot 鉴权凭证请求头

        参数:
            villa_id: 大别野ID

        返回:
            Dict[str, str]: 请求头
        """
        return {
            "x-rpc-bot_id": self.self_id,
            "x-rpc-bot_secret": self.bot_secret,
            "x-rpc-bot_villa_id": str(villa_id or ""),
        }

    @overrides(BaseBot)
    async def send(
        self,
        event: Event,
        message: Union[str, Message, MessageSegment],
        mention_sender: bool = False,
        reply_message: bool = False,
        **kwargs: Any,
    ) -> str:
        """发送消息

        参数:
            event: 事件
            message: 消息
            mention_sender: 是否@消息发送者. 默认为 False.
            reply_message: 是否引用原消息. 默认为 False.

        异常:
            RuntimeError: 事件不是消息事件时抛出

        返回:
            str: 消息ID
        """
        if not isinstance(event, SendMessageEvent):
            raise RuntimeError("Event cannot be replied to!")
        message = MessageSegment.text(message) if isinstance(message, str) else message
        message = message if isinstance(message, Message) else Message(message)
        if mention_sender:
            message.insert(
                0, MessageSegment.mention_user(event.villa_id, event.from_user_id)
            )
        if reply_message:
            message += MessageSegment.quote(event.msg_uid, event.send_at)
        content_info = await self.parse_message_content(message)
        return await self.send_message(
            villa_id=event.villa_id,
            room_id=event.room_id,
            object_name="MHY:Text",
            # object_name="MHY:Image" if content_info.content.images else "MHY:Text",
            msg_content=content_info.json(by_alias=True, exclude_none=True),
        )

    async def parse_message_content(self, message: Message) -> MessageContentInfo:
        """将适配器的Message对象转为大别野发送所需要的MessageContentInfo对象

        参数:
            message: 消息

        返回:
            MessageContentInfo: 消息内容对象
        """
        if quote := (message["quote"] or None):
            quote = quote[-1]
            quote = QuoteInfo(
                quoted_message_id=quote.data["msg_id"],
                quoted_message_send_time=quote.data["msg_send_time"],
                original_message_id=quote.data["msg_id"],
                original_message_send_time=quote.data["msg_send_time"],
            )

        message_text = ""
        message_offset = 0
        entities: List[TextEntity] = []
        images: List[Image] = []
        mentioned = MentionedInfo(type=MentionType.PART)
        for seg in message:
            if seg.type == "text":
                message_text += seg.data["text"]
                message_offset += len(seg.data["text"])
            elif seg.type == "mention_all":
                message_text += f"@{seg.data['show_text']} "
                entities.append(
                    TextEntity(
                        offset=message_offset,
                        length=6,
                        entity=MentionedAll(show_text=seg.data["show_text"]),
                    )
                )
                message_offset += 6
                mentioned.type = MentionType.ALL
            elif seg.type == "mentioned_robot":
                message_text += f"@{seg.data['bot_name']} "
                entities.append(
                    TextEntity(
                        offset=message_offset,
                        length=len(f"@{seg.data['bot_name']}".encode("utf-16")) // 2,
                        entity=MentionedRobot(
                            bot_id=seg.data["bot_id"], bot_name=seg.data["bot_name"]
                        ),
                    )
                )
                message_offset += len(f"@{seg.data['bot_name']}") + 1
                mentioned.user_id_list.append(seg.data["bot_id"])
            elif seg.type == "mentioned_user":
                # 需要调用API获取被@的用户的昵称
                user = await self.get_member(
                    villa_id=seg.data["villa_id"], uid=seg.data["user_id"]
                )
                message_text += f"@{user.basic.nickname} "
                entities.append(
                    TextEntity(
                        offset=message_offset,
                        length=len(f"@{user.basic.nickname}".encode("utf-16")) // 2,
                        entity=MentionedUser(
                            user_id=str(user.basic.uid), user_name=user.basic.nickname
                        ),
                    )
                )
                message_offset += len(f"@{user.basic.nickname}") + 1
                mentioned.user_id_list.append(str(user.basic.uid))
            elif seg.type == "villa_room_link":
                # 需要调用API获取房间的名称
                room = await self.get_room(
                    villa_id=seg.data["villa_id"], room_id=seg.data["room_id"]
                )
                message_text += f"#{room.room_name} "
                entities.append(
                    TextEntity(
                        offset=message_offset,
                        length=len(f"#{room.room_name}".encode("utf-16")) // 2,
                        entity=VillaRoomLink(
                            villa_id=str(seg.data["villa_id"]),
                            room_id=str(seg.data["room_id"]),
                            room_name=room.room_name,
                        ),
                    )
                )
                message_offset += len(f"#{room.room_name} ")
            elif seg.type == "link":
                message_text += seg.data["show_text"]
                entities.append(
                    TextEntity(
                        offset=message_offset,
                        length=len(seg.data["show_text"].encode("utf-16")) // 2,
                        entity=Link(
                            url=seg.data["url"], show_text=seg.data["show_text"]
                        ),
                    )
                )
                message_offset += len(seg.data["show_text"]) + 1
            elif seg.type == "image":
                images.append(
                    Image(
                        url=seg.data["url"],
                        size=ImageSize(
                            width=seg.data["width"], height=seg.data["height"]
                        )
                        if seg.data["width"] and seg.data["height"]
                        else None,
                        file_size=seg.data["file_size"],
                    )
                )

        # 不能单独只发图片而没有其他文本内容，塞一个零宽度空格
        if images and not message_text:
            message_text = "\u200B"

        if not (mentioned.type == MentionType.ALL and mentioned.user_id_list):
            mentioned = None
        return MessageContentInfo(
            content=MessageContent(text=message_text, entities=entities, images=images),
            mentionedInfo=mentioned,
            quote=quote,  # type: ignore
        )



from typing import Optional

from nonebot.exception import AdapterException
from nonebot.exception import ActionFailed as BaseActionFailed
from nonebot.exception import NetworkError as BaseNetworkError
from nonebot.exception import NoLogException as BaseNoLogException
from nonebot.exception import ApiNotAvailable as BaseApiNotAvailable

from .api import ApiResponse


class VillaAdapterException(AdapterException):
    def __init__(self):
        super().__init__("Villa")


class NoLogException(BaseNoLogException, VillaAdapterException):
    pass


class ActionFailed(BaseActionFailed, VillaAdapterException):
    def __init__(self, status_code: int, response: ApiResponse):
        self.status_code = status_code
        self.response = response

    def __repr__(self) -> str:
        return (
            f"<ActionFailed: {self.status_code}, retcode={self.response.retcode}, "
            f"message={self.response.message}, data={self.response.data}>"
        )

    def __str__(self):
        return self.__repr__()


class UnauthorizedException(ActionFailed):
    pass


# class RateLimitException(ActionFailed):
#     pass


class NetworkError(BaseNetworkError, VillaAdapterException):
    def __init__(self, msg: Optional[str] = None):
        super().__init__()
        self.msg: Optional[str] = msg
        """错误原因"""

    def __repr__(self):
        return f"<NetWorkError message={self.msg}>"

    def __str__(self):
        return self.__repr__()


class ApiNotAvailable(BaseApiNotAvailable, VillaAdapterException):
    def __init__(self, api: str):
        super().__init__()
        self.api = api

    def __repr__(self):
        return f"<ApiNotAvailable api={self.api}>"

    def __str__(self):
        return self.__repr__()



import json
import asyncio
from typing import Any, cast

from nonebot.typing import overrides
from nonebot.utils import escape_tag
from nonebot.drivers import (
    URL,
    Driver,
    Request,
    Response,
    ForwardDriver,
    ReverseDriver,
    HTTPServerSetup,
)

from nonebot.adapters import Adapter as BaseAdapter

from .bot import Bot
from .utils import log
from .config import Config
from .event import event_classes
from .api import API_HANDLERS, Payload
from .exception import ApiNotAvailable


class Adapter(BaseAdapter):
    @overrides(BaseAdapter)
    def __init__(self, driver: Driver, **kwargs: Any):
        super().__init__(driver, **kwargs)
        self.villa_config: Config = Config(**self.config.dict())
        self.base_url: URL = URL("https://bbs-api.miyoushe.com")
        self._setup()

    @classmethod
    @overrides(BaseAdapter)
    def get_name(cls) -> str:
        return "大别野"

    def _setup(self):
        # ReverseDriver用于接收回调事件，ForwardDriver用于调用API
        if not (
            isinstance(self.driver, ReverseDriver)
            and isinstance(self.driver, ForwardDriver)
        ):
            raise RuntimeError(
                f"Current driver {self.config.driver} doesn't support connections!"
                "Villa Adapter need a ReverseDriver and ForwardDriver to work."
            )
        for bot_info in self.villa_config.villa_bots:
            # TODO: 在启动时就先将Bot添加到bots中，而不是在收到事件时再添加
            http_setup = HTTPServerSetup(
                URL(bot_info.callback_url),
                "POST",
                f"大别野 {bot_info.bot_id} HTTP",
                self._handle_http,
            )
            self.setup_http_server(http_setup)

    async def _handle_http(self, request: Request) -> Response:
        if data := request.content:
            json_data = json.loads(data)
            if payload_data := json_data.get("event"):
                payload = Payload.parse_obj(payload_data)
                bot_id = payload.robot.template.id
                if (bot := self.bots.get(bot_id, None)) is None:
                    if (
                        bot_secret := next(
                            (
                                bot.bot_secret
                                for bot in self.villa_config.villa_bots
                                if bot.bot_id == bot_id
                            ),
                            None,
                        )
                    ) is not None:
                        bot = Bot(self, bot_id, payload.robot, bot_secret=bot_secret)
                        self.bot_connect(bot)
                        log("INFO", f"<y>Bot {escape_tag(bot.self_id)} connected</y>")
                    else:
                        log("WARNING", f"<r>Missing bot secret for bot {bot_id}</r>")
                bot = cast(Bot, bot)
                bot._bot_info = payload.robot

                if (event_class := event_classes.get(payload.type, None)) and (
                    event_class.__type__.name in payload.extend_data["EventData"]
                ):
                    try:
                        event = event_class.parse_obj(
                            payload.extend_data["EventData"][event_class.__type__.name]
                        )
                    except Exception as e:
                        log(
                            "WARNING",
                            f"Failed to parse event {escape_tag(repr(payload))}",
                            e,
                        )
                    else:
                        asyncio.create_task(bot.handle_event(event))
                else:
                    log(
                        "INFO",
                        f"Unknown event type: {payload.type} data={payload.extend_data}",
                    )
                # (Path().cwd() / f'test_event_{payload.created_at}.json').write_text(json.dumps(json_data, indent=4, ensure_ascii=False), encoding='utf-8')
                return Response(
                    200,
                    content=json.dumps({"retcode": 0, "message": "NoneBot2 Get it!"}),
                )
            return Response(400, content="Invalid Request Body")
        return Response(400, content="Invalid Request Body")

    @overrides(BaseAdapter)
    async def _call_api(self, bot: Bot, api: str, **data: Any) -> Any:
        log("DEBUG", f"Calling API <y>{api}</y>")
        log("TRACE", f"With Data <y>{data}</y>")
        if (api_handler := API_HANDLERS.get(api)) is None:
            raise ApiNotAvailable(api)
        return await api_handler(self, bot, **data)



from typing import List, Union, Literal

from .models import *

class ApiClient:
    async def check_member_bot_access_token(
        self, *, villa_id: int, token: str
    ) -> CheckMemberBotAccessTokenReturn: ...
    async def get_villa(self, *, villa_id: int) -> Villa: ...
    async def get_member(self, *, uid: int) -> Member: ...
    async def get_villa_members(
        self, *, villa_id: int, offset: int, size: int
    ) -> MemberListReturn: ...
    async def delete_villa_member(self, *, villa_id: int, uid: int) -> None: ...
    async def pin_message(
        self,
        *,
        villa_id: int,
        msg_uid: str,
        is_cancel: bool,
        room_id: int,
        send_at: int,
    ) -> None: ...
    async def recall_message(
        self, *, villa_id: int, msg_uid: str, room_id: int, msg_time: int
    ) -> None: ...
    async def send_message(
        self,
        *,
        villa_id: int,
        room_id: int,
        object_name: str,
        msg_content: Union[str, MessageContentInfo],
    ) -> str: ...
    async def create_group(self, *, villa_id: int, group_name: str) -> int: ...
    async def edit_group(
        self, *, villa_id: int, group_id: int, group_name: str
    ) -> None: ...
    async def delete_group(self, *, villa_id: int, group_id: int) -> None: ...
    async def get_group_list(self, *, villa_id: int) -> List[Group]: ...
    async def sort_group_list(self, *, villa_id: int, group_ids: List[int]) -> None: ...
    async def create_room(
        self,
        *,
        villa_id: int,
        room_name: str,
        room_type: Union[Literal[1, 2, 3], CreateRoomType],
        group_id: int,
        room_default_notify_type: Union[Literal[1, 2], CreateRoomDefaultNotifyType],
        send_msg_auth_range: SendMsgAuthRange,
    ) -> Room: ...
    async def edit_room(
        self, *, villa_id: int, room_id: int, room_name: str
    ) -> None: ...
    async def delete_room(self, *, villa_id: int, room_id: int) -> None: ...
    async def get_room(self, *, villa_id: int, room_id: int) -> Room: ...
    async def get_villa_group_room_list(self, *, villa_id: int) -> GroupRoom: ...
    async def sort_room_list(
        self, *, villa_id: int, room_list: List[RoomSort]
    ) -> None: ...
    async def operate_member_to_role(
        self, *, villa_id: int, role_id: int, uid: int, is_add: bool
    ) -> None: ...
    async def create_member_role(
        self, *, villa_id: int, name: str, color: str, permissions: List[Permission]
    ) -> int: ...
    async def edit_member_role(
        self,
        *,
        villa_id: int,
        role_id: int,
        name: str,
        color: str,
        permissions: List[Permission],
    ) -> None: ...
    async def delete_member_role(self, *, villa_id: int, role_id: int) -> None: ...
    async def get_member_role(
        self, *, villa_id: int, role_id: int
    ) -> MemberRoleDetail: ...
    async def get_villa_member_roles(
        self, *, villa_id: int
    ) -> List[MemberRoleDetail]: ...
    async def get_all_emoticon(self) -> List[Emoticon]: ...
    async def audit(
        self,
        *,
        villa_id: int,
        audit_content: str,
        pass_through: str,
        room_id: int,
        uid: int,
    ) -> str: ...



from typing import List

from pydantic import Extra, Field, BaseModel


class BotInfo(BaseModel):
    bot_id: str
    bot_secret: str
    callback_url: str


class Config(BaseModel, extra=Extra.ignore):
    villa_bots: List[BotInfo] = Field(default_factory=list)



from enum import IntEnum
from typing import Dict, Type, Optional

from pydantic import root_validator
from nonebot.typing import overrides
from nonebot.utils import escape_tag

from nonebot.adapters import Event as BaseEvent

from .api import MessageContentInfo
from .message import Message, MessageSegment


class EventType(IntEnum):
    """事件类型"""

    JoinVilla = 1
    SendMessage = 2
    CreateRobot = 3
    DeleteRobot = 4
    AddQuickEmoticon = 5
    AuditCallback = 6


class AuditResult(IntEnum):
    """审核结果类型"""

    Compatibility = 0
    """兼容"""
    Pass = 1
    """通过"""
    Reject = 2
    """驳回"""


class Event(BaseEvent):
    """Villa 事件基类"""

    __type__: EventType

    @overrides(BaseEvent)
    def get_event_name(self) -> str:
        return self.__type__.name

    @overrides(BaseEvent)
    def get_event_description(self) -> str:
        return escape_tag(repr(self.dict()))

    @overrides(BaseEvent)
    def get_message(self):
        raise ValueError("Event has no message!")

    @overrides(BaseEvent)
    def get_user_id(self) -> str:
        raise ValueError("Event has no context!")

    @overrides(BaseEvent)
    def get_session_id(self) -> str:
        raise ValueError("Event has no context!")

    @overrides(BaseEvent)
    def is_tome(self) -> bool:
        return False


class NoticeEvent(Event):
    """通知事件"""

    @overrides(BaseEvent)
    def get_type(self) -> str:
        return "notice"


class MessageEvent(Event):
    """消息事件

    但目前大别野只有SendMessageEvent这一种消息事件，所以推荐直接使用SendMessageEvent"""

    @overrides(BaseEvent)
    def get_type(self) -> str:
        return "message"


class JoinVillaEvent(NoticeEvent):
    """新用户加入大别野事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html###JoinVilla"""

    __type__ = EventType.JoinVilla
    join_uid: int
    """用户ID"""
    join_user_nickname: str
    """用户昵称"""
    join_at: int
    """用户加入时间的时间戳"""

    @overrides(BaseEvent)
    def get_user_id(self) -> str:
        return str(self.join_uid)

    @overrides(BaseEvent)
    def get_session_id(self) -> str:
        return str(self.join_uid)


class SendMessageEvent(MessageEvent):
    """用户@机器人发送消息事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html###SendMessage"""

    __type__ = EventType.SendMessage
    content: MessageContentInfo
    """消息内容"""
    from_user_id: int
    """发送者ID"""
    send_at: int
    """发送时间的时间戳"""
    room_id: int
    """房间ID"""
    object_name: int
    """目前只支持文本类型消息"""
    nickname: str
    """用户昵称"""
    msg_uid: str
    """消息ID"""
    bot_msg_id: Optional[str]
    """如果被回复的消息从属于机器人，则该字段不为空字符串"""

    villa_id: int
    """大别野ID"""
    bot_id: str
    """BotID"""
    to_me: bool = False
    """是否和Bot有关"""
    message: Message
    """事件消息"""

    @overrides(BaseEvent)
    def get_message(self) -> Message:
        """获取事件消息"""
        return self.message

    @overrides(Event)
    def is_tome(self) -> bool:
        """是否和Bot有关"""
        return self.to_me

    @overrides(BaseEvent)
    def get_user_id(self) -> str:
        """获取用户ID"""
        return str(self.from_user_id)

    @overrides(BaseEvent)
    def get_session_id(self) -> str:
        """获取会话ID"""
        return f"{self.room_id}-{self.from_user_id}"

    @root_validator(pre=True)
    def _(cls, data: dict):
        msg = Message()
        msg_content_info = data["content"]
        if quote := msg_content_info.get("quote"):
            msg.append(
                MessageSegment.quote(
                    message_id=quote["quoted_message_id"],
                    message_send_time=quote["quoted_message_send_time"],
                )
            )

        content = msg_content_info["content"]
        text = content["text"]
        entities = content["entities"]
        if not entities:
            return Message(MessageSegment.text(text))
        text = text.encode("utf-16")
        last_offset: int = 0
        last_length: int = 0
        for entity in entities:
            end_offset: int = last_offset + last_length
            offset: int = entity["offset"]
            length: int = entity["length"]
            entity_detail = entity["entity"]
            if offset != end_offset:
                msg.append(
                    MessageSegment.text(
                        text[((end_offset + 1) * 2) : ((offset + 1) * 2)].decode(
                            "utf-16"
                        )
                    )
                )
            entity_text = text[(offset + 1) * 2 : (offset + length + 1) * 2].decode(
                "utf-16"
            )
            if entity_detail["type"] == "mentioned_robot":
                entity_detail["bot_name"] = entity_text.lstrip("@")[:-1]
                msg.append(
                    MessageSegment.mention_robot(
                        entity_detail["bot_id"], entity_detail["bot_name"]
                    )
                )
            elif entity_detail["type"] == "mentioned_user":
                entity_detail["user_name"] = entity_text.lstrip("@")[:-1]
                msg.append(
                    MessageSegment.mention_user(
                        int(entity_detail["user_id"]), data["villa_id"]
                    )
                )
            elif entity_detail["type"] == "mention_all":
                entity_detail["show_text"] = entity_text.lstrip("@")[:-1]
                msg.append(MessageSegment.mention_all(entity_detail["show_text"]))
            elif entity_detail["type"] == "villa_room_link":
                entity_detail["room_name"] = entity_text.lstrip("#")[:-1]
                msg.append(
                    MessageSegment.room_link(
                        int(entity_detail["villa_id"]),
                        int(entity_detail["room_id"]),
                    )
                )
            else:
                entity_detail["show_text"] = entity_text
                msg.append(MessageSegment.link(entity_detail["url"], entity_text))
            last_offset = offset
            last_length = length
        end_offset = last_offset + last_length
        if last_text := text[(end_offset + 1) * 2 :].decode("utf-16"):
            msg.append(MessageSegment.text(last_text))
        data["message"] = msg
        return data


class CreateRobotEvent(NoticeEvent):
    """大别野添加机器人实例事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html###CreateRobot"""

    __type__ = EventType.CreateRobot
    villa_id: int
    """大别野ID"""


class DeleteRobotEvent(NoticeEvent):
    """大别野删除机器人实例事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html###DeleteRobot"""

    __type__ = EventType.DeleteRobot
    villa_id: int
    """大别野ID"""


class AddQuickEmoticonEvent(NoticeEvent):
    """用户使用表情回复消息表态事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html#AddQuickEmoticon"""

    __type__ = EventType.AddQuickEmoticon
    villa_id: int
    """大别野ID"""
    room_id: int
    """房间ID"""
    uid: int
    """发送表情的用户ID"""
    emoticon_id: int
    """表情ID"""
    emoticon: str
    """表情内容"""
    msg_uid: str
    """被回复的消息 id"""
    bot_msg_id: Optional[str]
    """如果被回复的消息从属于机器人，则该字段不为空字符串"""
    is_cancel: bool = False
    """是否是取消表情"""


class AuditCallbackEvent(NoticeEvent):
    """审核结果回调事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html#AuditCallback"""

    __type__ = EventType.AuditCallback
    audit_id: str
    """审核事件 id"""
    bot_tpl_id: str
    """机器人 id"""
    villa_id: int
    """大别野 ID"""
    room_id: int
    """房间 id（和审核接口调用方传入的值一致）"""
    user_id: int
    """用户 id（和审核接口调用方传入的值一致）"""
    pass_through: str
    """透传数据（和审核接口调用方传入的值一致）"""
    audit_result: AuditResult
    """审核结果"""


event_classes: Dict[int, Type[Event]] = {
    EventType.JoinVilla.value: JoinVillaEvent,
    EventType.SendMessage.value: SendMessageEvent,
    EventType.CreateRobot.value: CreateRobotEvent,
    EventType.DeleteRobot.value: DeleteRobotEvent,
    EventType.AddQuickEmoticon.value: AddQuickEmoticonEvent,
    EventType.AuditCallback.value: AuditCallbackEvent,
}

__all__ = [
    "Event",
    "NoticeEvent",
    "MessageEvent",
    "JoinVillaEvent",
    "SendMessageEvent",
    "CreateRobotEvent",
    "DeleteRobotEvent",
    "AddQuickEmoticonEvent",
    "AuditCallbackEvent",
]



from typing import TYPE_CHECKING, List, Union, Literal

from pydantic import parse_obj_as
from nonebot.drivers import Request

from .models import *
from .request import _request

if TYPE_CHECKING:
    from ..bot import Bot
    from ..adapter import Adapter


async def _check_member_bot_access_token(
    adapter: "Adapter", bot: "Bot", villa_id: int, token: str
) -> CheckMemberBotAccessTokenReturn:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/checkMemberBotAccessToken",
        headers=bot.get_authorization_header(
            villa_id
        ),  # TODO: 文档中说无需鉴权，但不带header会报错，有待确认
        json={"token": token},
    )
    return parse_obj_as(
        CheckMemberBotAccessTokenReturn, await _request(adapter, bot, request)
    )


async def _get_villa(adapter: "Adapter", bot: "Bot", villa_id: int) -> Villa:
    request = Request(
        method="GET",
        url=adapter.base_url / "vila/api/bot/platform/getVilla",
        headers=bot.get_authorization_header(villa_id),
    )
    return parse_obj_as(Villa, (await _request(adapter, bot, request))["villa"])


async def _get_member(
    adapter: "Adapter", bot: "Bot", villa_id: int, uid: int
) -> Member:
    request = Request(
        method="GET",
        url=adapter.base_url / "vila/api/bot/platform/getMember",
        headers=bot.get_authorization_header(villa_id),
        json={"uid": uid},
    )
    return parse_obj_as(Member, (await _request(adapter, bot, request))["member"])


async def _get_villa_members(
    adapter: "Adapter", bot: "Bot", villa_id: int, offset: int, size: int
) -> MemberListReturn:
    request = Request(
        method="GET",
        url=adapter.base_url / "vila/api/bot/platform/getVillaMembers",
        headers=bot.get_authorization_header(villa_id),
        json={"offset": offset, "size": size},
    )
    return parse_obj_as(MemberListReturn, await _request(adapter, bot, request))


async def _delete_villa_member(
    adapter: "Adapter", bot: "Bot", villa_id: int, uid: int
) -> None:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/deleteVillaMember",
        headers=bot.get_authorization_header(villa_id),
        json={"uid": uid},
    )
    await _request(adapter, bot, request)


async def _pin_message(
    adapter: "Adapter",
    bot: "Bot",
    villa_id: int,
    msg_uid: str,
    is_cancel: bool,
    room_id: int,
    send_at: int,
) -> None:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/pinMessage",
        headers=bot.get_authorization_header(villa_id),
        json={
            "msg_uid": msg_uid,
            "is_cancel": is_cancel,
            "room_id": room_id,
            "send_at": send_at,
        },
    )
    await _request(adapter, bot, request)


async def _recall_message(
    adapter: "Adapter",
    bot: "Bot",
    villa_id: int,
    msg_uid: str,
    room_id: int,
    msg_time: int,
) -> None:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/recallMessage",
        headers=bot.get_authorization_header(villa_id),
        json={"msg_uid": msg_uid, "room_id": room_id, "msg_time": msg_time},
    )
    await _request(adapter, bot, request)


async def _send_message(
    adapter: "Adapter",
    bot: "Bot",
    villa_id: int,
    room_id: int,
    object_name: str,
    msg_content: Union[str, MessageContentInfo],
) -> str:
    if isinstance(msg_content, MessageContentInfo):
        content = msg_content.json(by_alias=True, exclude_none=True)
    else:
        content = msg_content
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/sendMessage",
        headers=bot.get_authorization_header(villa_id),
        json={"room_id": room_id, "object_name": object_name, "msg_content": content},
    )
    return (await _request(adapter, bot, request))["bot_msg_id"]


async def _create_group(
    adapter: "Adapter", bot: "Bot", villa_id: int, group_name: str
) -> int:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/createGroup",
        headers=bot.get_authorization_header(villa_id),
        json={"group_name": group_name},
    )
    return (await _request(adapter, bot, request))["group_id"]


async def _edit_group(
    adapter: "Adapter", bot: "Bot", villa_id: int, group_id: int, group_name: str
) -> None:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/editGroup",
        headers=bot.get_authorization_header(villa_id),
        json={"group_id": group_id, "group_name": group_name},
    )
    await _request(adapter, bot, request)


async def _delete_group(
    adapter: "Adapter", bot: "Bot", villa_id: int, group_id: int
) -> None:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/deleteGroup",
        headers=bot.get_authorization_header(villa_id),
        json={"group_id": group_id},
    )
    await _request(adapter, bot, request)


async def _get_group_list(adapter: "Adapter", bot: "Bot", villa_id: int) -> List[Group]:
    request = Request(
        method="GET",
        url=adapter.base_url / "vila/api/bot/platform/getGroupList",
        headers=bot.get_authorization_header(villa_id),
    )
    return parse_obj_as(List[Group], (await _request(adapter, bot, request))["list"])


async def _sort_group_list(
    adapter: "Adapter", bot: "Bot", villa_id: int, group_ids: List[int]
) -> None:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/sortGroupList",
        headers=bot.get_authorization_header(villa_id),
        json={"villa_id": villa_id, "group_ids": group_ids},
    )
    await _request(adapter, bot, request)


async def _create_room(
    adapter: "Adapter",
    bot: "Bot",
    villa_id: int,
    room_name: str,
    room_type: Union[Literal[1, 2, 3], CreateRoomType],
    group_id: int,
    room_default_notify_type: Union[Literal[1, 2], CreateRoomDefaultNotifyType],
    send_msg_auth_range: SendMsgAuthRange,
) -> Room:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/createRoom",
        headers=bot.get_authorization_header(villa_id),
        json={
            "room_name": room_name,
            "room_type": room_type,
            "group_id": group_id,
            "room_default_notify_type": room_default_notify_type,
            "send_msg_auth_range": send_msg_auth_range.dict(),
        },
    )
    return parse_obj_as(Room, (await _request(adapter, bot, request))["room"])


async def _edit_room(
    adapter: "Adapter", bot: "Bot", villa_id: int, room_id: int, room_name: str
) -> None:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/editRoom",
        headers=bot.get_authorization_header(villa_id),
        json={"room_id": room_id, "room_name": room_name},
    )
    await _request(adapter, bot, request)


async def _delete_room(
    adapter: "Adapter", bot: "Bot", villa_id: int, room_id: int
) -> None:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/deleteRoom",
        headers=bot.get_authorization_header(villa_id),
        json={"room_id": room_id},
    )
    await _request(adapter, bot, request)


async def _get_room(
    adapter: "Adapter", bot: "Bot", villa_id: int, room_id: int
) -> Room:
    request = Request(
        method="GET",
        url=adapter.base_url / "vila/api/bot/platform/getRoom",
        headers=bot.get_authorization_header(villa_id),
        json={"room_id": room_id},
    )
    return parse_obj_as(Room, (await _request(adapter, bot, request))["room"])


async def _get_villa_group_room_list(
    adapter: "Adapter", bot: "Bot", villa_id: int
) -> GroupRoom:
    request = Request(
        method="GET",
        url=adapter.base_url / "vila/api/bot/platform/getVillaGroupRoomList",
        headers=bot.get_authorization_header(villa_id),
    )
    return parse_obj_as(GroupRoom, (await _request(adapter, bot, request))["list"])


async def _sort_room_list(
    adapter: "Adapter", bot: "Bot", villa_id: int, room_list: List[RoomSort]
) -> None:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/sortRoomList",
        headers=bot.get_authorization_header(villa_id),
        json={"villa_id": villa_id, "room_list": [room.dict() for room in room_list]},
    )
    await _request(adapter, bot, request)


async def _operate_member_to_role(
    adapter: "Adapter", bot: "Bot", villa_id: int, role_id: int, uid: int, is_add: bool
) -> None:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/operateMemberToRole",
        headers=bot.get_authorization_header(villa_id),
        json={"role_id": role_id, "uid": uid, "is_add": is_add},
    )
    await _request(adapter, bot, request)


async def _create_member_role(
    adapter: "Adapter",
    bot: "Bot",
    villa_id: int,
    name: str,
    color: Color,
    permissions: List[Permission],
) -> int:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/createMemberRole",
        headers=bot.get_authorization_header(villa_id),
        json={"name": name, "color": str(color), "permissions": permissions},
    )
    return (await _request(adapter, bot, request))["id"]


async def _edit_member_role(
    adapter: "Adapter",
    bot: "Bot",
    villa_id: int,
    role_id: int,
    name: str,
    color: Color,
    permissions: List[Permission],
) -> None:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/editMemberRole",
        headers=bot.get_authorization_header(villa_id),
        json={
            "id": role_id,
            "name": name,
            "color": str(color),
            "permissions": permissions,
        },
    )
    await _request(adapter, bot, request)


async def _delete_member_role(
    adapter: "Adapter", bot: "Bot", villa_id: int, role_id: int
) -> None:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/deleteMemberRole",
        headers=bot.get_authorization_header(villa_id),
        json={"id": role_id},
    )
    await _request(adapter, bot, request)


async def _get_member_role_info(
    adapter: "Adapter", bot: "Bot", villa_id: int, role_id: int
) -> MemberRoleDetail:
    request = Request(
        method="GET",
        url=adapter.base_url / "vila/api/bot/platform/getMemberRoleInfo",
        headers=bot.get_authorization_header(villa_id),
        json={"id": role_id},
    )
    return parse_obj_as(
        MemberRoleDetail, (await _request(adapter, bot, request))["role"]
    )


async def _get_villa_member_roles(
    adapter: "Adapter", bot: "Bot", villa_id: int
) -> List[MemberRoleDetail]:
    request = Request(
        method="GET",
        url=adapter.base_url / "vila/api/bot/platform/getVillaMemberRoles",
        headers=bot.get_authorization_header(villa_id),
    )
    return parse_obj_as(
        List[MemberRoleDetail], (await _request(adapter, bot, request))["list"]
    )


async def _get_all_emoticons(adapter: "Adapter", bot: "Bot") -> List[Emoticon]:
    request = Request(
        method="GET", url=adapter.base_url / "vila/api/bot/platform/getAllEmoticons"
    )
    return parse_obj_as(List[Emoticon], (await _request(adapter, bot, request))["list"])


async def _audit(
    adapter: "Adapter",
    bot: "Bot",
    villa_id: int,
    audit_content: str,
    pass_through: str,
    room_id: int,
    uid: int,
) -> str:
    request = Request(
        method="POST",
        url=adapter.base_url / "vila/api/bot/platform/audit",
        headers=bot.get_authorization_header(villa_id),
        json={
            "audit_content": audit_content,
            "pass_through": pass_through,
            "room_id": room_id,
            "uid": uid,
        },
    )
    return (await _request(adapter, bot, request))["audit_id"]


API_HANDLERS = {
    "check_member_bot_access_token": _check_member_bot_access_token,
    "get_villa": _get_villa,
    "get_member": _get_member,
    "get_villa_members": _get_villa_members,
    "delete_villa_member": _delete_villa_member,
    "pin_message": _pin_message,
    "recall_message": _recall_message,
    "send_message": _send_message,
    "create_group": _create_group,
    "edit_group": _edit_group,
    "delete_group": _delete_group,
    "get_group_list": _get_group_list,
    "sort_group_list": _sort_group_list,
    "create_room": _create_room,
    "edit_room": _edit_room,
    "delete_room": _delete_room,
    "get_room": _get_room,
    "get_villa_group_room_list": _get_villa_group_room_list,
    "sort_room_list": _sort_room_list,
    "operate_member_to_role": _operate_member_to_role,
    "create_member_role": _create_member_role,
    "edit_member_role": _edit_member_role,
    "delete_member_role": _delete_member_role,
    "get_member_role_info": _get_member_role_info,
    "get_villa_member_roles": _get_villa_member_roles,
    "get_all_emoticons": _get_all_emoticons,
    "audit": _audit,
}



from typing import Type, Union, Iterable, Optional

from nonebot.typing import overrides
from nonebot.utils import escape_tag

from nonebot.adapters import Message as BaseMessage
from nonebot.adapters import MessageSegment as BaseMessageSegment


class MessageSegment(BaseMessageSegment["Message"]):
    @classmethod
    @overrides(BaseMessageSegment)
    def get_message_class(cls) -> Type["Message"]:
        return Message

    @overrides(BaseMessageSegment)
    def __add__(
        self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]
    ) -> "Message":
        return super().__add__(other)

    @overrides(BaseMessageSegment)
    def __radd__(
        self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]
    ) -> "Message":
        return super().__radd__(other)

    @overrides(BaseMessageSegment)
    def is_text(self) -> bool:
        return self.type == "text"

    @staticmethod
    def text(text: str) -> "TextSegment":
        """纯文本消息段

        参数:
            text: 文本内容

        返回:
            TextSegment: 消息段对象
        """
        return TextSegment("text", {"text": text})

    @staticmethod
    def mention_robot(bot_id: str, bot_name: str) -> "MentionRobotSegement":
        """@机器人消息段

        参数:
            bot_id: 机器人ID
            bot_name: 机器人的名字

        返回:
            MentionRobotSegement: 消息段对象
        """
        return MentionRobotSegement(
            "mentioned_robot", {"bot_id": bot_id, "bot_name": bot_name}
        )

    @staticmethod
    def mention_user(villa_id: int, user_id: int) -> "MentionUserSegement":
        """@用户消息段

        参数:
            villa_id: 用户所在大别野ID
            user_id: 用户ID

        返回:
            MentionUserSegement: 消息段对象
        """
        return MentionUserSegement(
            "mentioned_user", {"villa_id": villa_id, "user_id": user_id}
        )

    @staticmethod
    def mention_all(show_text: str = "全体成员") -> "MentionAllSegement":
        """@全体成员消息段"""
        return MentionAllSegement("mention_all", {"show_text": show_text})

    @staticmethod
    def room_link(villa_id: int, room_id: int) -> "RoomLinkSegment":
        """房间链接消息段，点击后可以跳转到指定房间

        参数:
            villa_id: 大别野ID
            room_id: 房间ID

        返回:
            VillaRoomLinkSegment: 消息段对象
        """
        return RoomLinkSegment(
            "villa_room_link", {"villa_id": villa_id, "room_id": room_id}
        )

    @staticmethod
    def link(url: str, show_text: Optional[str] = None) -> "LinkSegment":
        """链接消息段，使用该消息段才能让链接可以直接点击进行跳转

        参数:
            url: 链接
            show_text: 链接显示的文本

        返回:
            LinkSegment: 消息段对象
        """
        return LinkSegment("link", {"url": url, "show_text": show_text or url})

    @staticmethod
    def quote(message_id: str, message_send_time: int) -> "QuoteSegment":
        """引用(回复)消息段

        参数:
            message_id: 被引用的消息ID
            message_send_time: 被引用的消息发送时间

        返回:
            QuoteSegment: 消息段对象
        """
        return QuoteSegment(
            "quote", {"msg_id": message_id, "msg_send_time": message_send_time}
        )

    @staticmethod
    def image(
        url: str,
        width: Optional[int] = None,
        height: Optional[int] = None,
        file_size: Optional[int] = None,
    ) -> "ImageSegment":
        """图片消息段

        参数:
            url: 图片链接
            width: 图片宽度
            height: 图片高度
            file_size: 图片大小

        返回:
            ImageSegment: 消息段对象
        """
        return ImageSegment(
            "image",
            {"url": url, "width": width, "height": height, "file_size": file_size},
        )


class TextSegment(MessageSegment):
    @overrides(MessageSegment)
    def __str__(self) -> str:
        return escape_tag(self.data["text"])


class MentionRobotSegement(MessageSegment):
    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"<MentionRobot:bot_id={self.data['bot_id']} bot_name={self.data['bot_name']}>"


class MentionUserSegement(MessageSegment):
    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"<MentionUser:user_id={self.data['user_id']}>"


class MentionAllSegement(MessageSegment):
    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"<MentionAll:show_text={self.data['show_text']}>"


class RoomLinkSegment(MessageSegment):
    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"<RoomLink:villa_id={self.data['villa_id']} room_id={self.data['room_id']}>"


class LinkSegment(MessageSegment):
    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"Link:url={self.data['url']}"


class ImageSegment(MessageSegment):
    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"<Image:url={self.data['url']}>"


class QuoteSegment(MessageSegment):
    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"<Quote:msg_id={self.data['msg_id']}>"


class Message(BaseMessage[MessageSegment]):
    @classmethod
    @overrides(BaseMessage)
    def get_segment_class(cls) -> Type[MessageSegment]:
        return MessageSegment

    @overrides(BaseMessage)
    def __add__(
        self, other: Union[str, MessageSegment, Iterable[MessageSegment]]
    ) -> "Message":
        return super(Message, self).__add__(
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @overrides(BaseMessage)
    def __radd__(
        self, other: Union[str, MessageSegment, Iterable[MessageSegment]]
    ) -> "Message":
        return super(Message, self).__radd__(
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @staticmethod
    @overrides(BaseMessage)
    def _construct(msg: str) -> Iterable[MessageSegment]:
        yield MessageSegment.text(msg)
